package com.expd.apm.extension.agent.inputargs;

import java.util.Map;

import com.expd.apm.extension.agent.metricpublisher.MetricPublisherFactory;
import com.wily.introscope.agent.IAgent;
import com.wily.introscope.agent.service.IAgentService;
import com.wily.util.feedback.Module;
import com.wily.util.properties.IndexedProperties;
import com.wily.wilyassert.Assertion;

public class InputArgsAgentService implements IAgentService {
	public static final String kInputArgsEnabledKey = "introscope.agent.inputargs.enable";
	public static final String kInputArgsPollerIntervalKey = "introscope.agent.inputargs.pollerinterval";
	private static final String kINPUTARGS_NAME = "InputArgs Poller";
	private long kInputArgsPollInMillis;
	private IndexedProperties kInputArgsProps;
	private static final Module kInputArgsServiceModule = new Module("InputArgsAgentService");
	private static IAgent fAgent;
	//private static volatile InputArgsEnabledConfigurationProperty fInputArgsEnabledConfigurationProperty;
	private InputArgs fInputArgs;
	boolean inputArgsOn = false;

	public int IAgentService_getServiceVersion() {
		return 2;
	}

	public void IAgentService_startService(IAgent agent, @SuppressWarnings("rawtypes") Map paramMap)
	throws Exception {
		fAgent = agent;
		kInputArgsProps = agent.IAgent_getIndexedProperties();
		Assertion.wilyAssert(kInputArgsProps != null);
		
		inputArgsOn = kInputArgsProps.getBooleanProperty(kInputArgsEnabledKey, false);
		kInputArgsPollInMillis = kInputArgsProps.getLongProperty(kInputArgsPollerIntervalKey, 600000L);
		
		if (!(inputArgsOn)) {
			fAgent.IAgent_getModuleFeedback().verbose(kInputArgsServiceModule, "InputArgs Extension disabled");
			return;
		}
		try {
			MetricPublisherFactory.getInstance().useAgentMetricPublisher(fAgent);
			fAgent.IAgent_getModuleFeedback().verbose(kInputArgsServiceModule, "Activating InputArgs Extension");
			initialize();
			fAgent.IAgent_getModuleFeedback().info(kInputArgsServiceModule, "InputArgs Extension activated");
		} catch (Exception e) {
			agent.IAgent_safeReportError("Failed to activate debug mode", e);
		}
	}

	private void initialize() {
		synchronized (InputArgsAgentService.class) {
			if (this.fInputArgs == null) {
				this.fInputArgs = new InputArgs(fAgent);

				if (this.fInputArgs.getCollector() != null) {
					fAgent.IAgent_getCommonHeartbeat().addBehavior(this.fInputArgs, kINPUTARGS_NAME, true, kInputArgsPollInMillis, false, 0);
					fAgent.IAgent_getModuleFeedback().info(kInputArgsServiceModule, "InputArgs heartbeat behavior added.");
				} else {
					fAgent.IAgent_getModuleFeedback().verbose(kInputArgsServiceModule, "Could not add InputArgs heartbeat behavior.");
				}
			} else {
				fAgent.IAgent_getModuleFeedback().verbose(kInputArgsServiceModule, "InputArgs Collector not initialized.");
			}
		}
	}
	/*
	public static boolean isInputArgsEnabled() {
		return fInputArgsEnabledConfigurationProperty.isEnabled();
	}

	private final class InputArgsEnabledConfigurationProperty extends BooleanConfigurationProperty {
		volatile boolean value;

		private InputArgsEnabledConfigurationProperty(IAgent paramIAgent) {
			super(InputArgsAgentService.kInputArgsEnabledKey, Boolean.FALSE, paramIAgent.IAgent_getModuleFeedback(), InputArgsAgentService.kInputArgsServiceModule, paramIAgent.IAgent_getStringLocalizer());
		}

		public void set(Object newValue) {
			this.value = ((Boolean)newValue).booleanValue();
			if (this.value) {
				InputArgsAgentService.this.initialize();
			} else {
				synchronized (InputArgsAgentService.class) {
					if (InputArgsAgentService.this.fInputArgs != null) {
						if (InputArgsAgentService.this.fBehavior != null) {
							InputArgsAgentService.this.fInputArgs.turnOffAllMetrics();
							InputArgsAgentService.fAgent.IAgent_getCommonHeartbeat().removeBehavior(InputArgsAgentService.this.fBehavior);
							InputArgsAgentService.fAgent.IAgent_getModuleFeedback().info(InputArgsAgentService.kInputArgsServiceModule, Messages.getString("InputArgsExtn.StopMessage"));
						}

						//InputArgsAgentService.access$302(InputArgsAgentService.this);
					}
				}
			}
		}

		public final boolean isEnabled() {
			return this.value;
		}
	}
	 */

}
