package com.expd.apm.extension.agent.inputargs;

import java.lang.management.ManagementFactory;
import java.lang.management.RuntimeMXBean;
import java.util.List;

import com.expd.apm.extension.agent.metricpublisher.IMetricPublisher;
import com.expd.apm.extension.agent.metricpublisher.MetricPublisherFactory;
import com.wily.introscope.agent.IAgent;
import com.wily.introscope.agent.stat.DataAccumulatorFactory;
import com.wily.introscope.spec.metric.BadlyFormedNameException;
import com.wily.introscope.spec.metric.MetricNotFoundException;
import com.wily.util.feedback.IModuleFeedbackChannel;
import com.wily.util.feedback.Module;

public class InputArgsCollector {
	protected IMetricPublisher metricPublisher = null;
	private RuntimeMXBean fRuntimeMXBean;
	private IModuleFeedbackChannel feedbackChannel;
	private Module kInputArgsCollector = new Module("InputArgsCollector");
	private String inputArgs = null;
	private String fInputArgsName;


	public InputArgsCollector(final String metricName, final IAgent agent) {
		this.feedbackChannel = agent.IAgent_getModuleFeedback();
		initPublisher();
		
		this.fRuntimeMXBean = ManagementFactory.getRuntimeMXBean();
		if (fRuntimeMXBean != null) {
			this.fInputArgsName = metricName;
			feedbackChannel.debug(this.kInputArgsCollector, "Collector ready to publish");
		}
	}

	public void publishMetricValues() {
		if (this.inputArgs == null) {
			if (this.fRuntimeMXBean != null) {	    
				this.feedbackChannel.debug(this.kInputArgsCollector, "Collector publishing new metric");
				final List<String> inputArgs = this.fRuntimeMXBean.getInputArguments();
				final StringBuilder args = new StringBuilder();
	
				for(final String arg : inputArgs) {
					args.append(arg);
					args.append(System.getProperty("line.separator"));
				}
				this.inputArgs = args.toString().trim();
				this.metricPublisher.publish( this.fInputArgsName, this.inputArgs );
			}
		} else {
			this.feedbackChannel.debug(this.kInputArgsCollector, "Collector publishing old metric");
			this.metricPublisher.publish( this.fInputArgsName, this.inputArgs );
		}
	}

	public void turnOffMetrics(DataAccumulatorFactory factory) throws BadlyFormedNameException, MetricNotFoundException {
		factory.removeMetric(this.fInputArgsName);
	}

	protected void initPublisher() {
		this.metricPublisher = MetricPublisherFactory.getInstance().getMetricPublisher();
	}
}
