package com.expd.apm.extension.agent.metricpublisher;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import com.wily.introscope.agent.IAgent;
import com.wily.introscope.agent.stat.DataAccumulatorFactory;
import com.wily.introscope.agent.stat.IIntegerAverageDataAccumulator;
import com.wily.introscope.agent.stat.IIntegerFluctuatingCounterDataAccumulator;
import com.wily.introscope.agent.stat.ILongFluctuatingCounterDataAccumulator;
import com.wily.introscope.agent.stat.IStringConstantDataAccumulator;
import com.wily.util.feedback.IModuleFeedbackChannel;
import com.wily.util.feedback.Module;

/**
 * The Class AgentMetricPublisher; used to output various metrics to the Introscope Workstation (via the APM cluster). 
 *
 * @author Jack Butler (jack.butler@expeditors.com)
 * @version 1.0 2013-02-11
 */
public class AgentMetricPublisher implements IMetricPublisher {
	private Map<String, Long> metrics = new HashMap<String, Long>();
	public String metricName = null;
	public long lastUpdate = 0L;
	private IModuleFeedbackChannel feedbackChannel;	
	private Module kMetricPublisher = new Module("AgentMetricPublisher");
	private DataAccumulatorFactory accumulatorFactory = null;
	private IIntegerAverageDataAccumulator intPercentDataAccumulator = null;
	private IIntegerFluctuatingCounterDataAccumulator intFluctCountDataAccumulator = null;
	private ILongFluctuatingCounterDataAccumulator longFluctCountDataAccumulator = null;
	private IStringConstantDataAccumulator stringConstantDataAccumulator = null;

	private boolean isPercentage = false;

	/**
	 * Instantiates a new agent metric publisher.
	 *
	 * @param agent the agent
	 */
	public AgentMetricPublisher(IAgent agent) {
		this.accumulatorFactory = agent.IAgent_getDataAccumulatorFactory();
		this.feedbackChannel = agent.IAgent_getModuleFeedback();
	}

	public AgentMetricPublisher(IAgent agent, boolean isPercentage) {
		this.accumulatorFactory = agent.IAgent_getDataAccumulatorFactory();
		this.isPercentage = isPercentage;
		this.feedbackChannel = agent.IAgent_getModuleFeedback();
	}

	public boolean isPercentage() {
		return this.isPercentage;
	}

	public void setPercentage(boolean isPercentage) {
		this.isPercentage = isPercentage;
	}

	private void checkName(String newMetricName) {
		if(!newMetricName.equals(this.metricName)) {
			//if(this.metricName != null) removeMetric(this.metricName);
			//if(!this.metrics.containsKey(newMetricName)) metrics.put(newMetricName, System.currentTimeMillis());
			this.metricName = newMetricName;
		}
	}	

	/* (non-Javadoc)
	 * @see com.expd.apm.extension.jtop.metricpublisher.IMetricPublisher#publish(java.lang.String, int)
	 */
	public void publish(String newMetricName, int value) {
		checkName(newMetricName);
		if(!isPercentage) {
			if(this.intFluctCountDataAccumulator == null ) {
				this.intFluctCountDataAccumulator = this.accumulatorFactory.safeGetIntegerFluctuatingCounterDataAccumulator(metricName);
			}
			this.intFluctCountDataAccumulator.IIntegerCounterDataAccumulator_setValue(value);
		} else {
			if(this.intPercentDataAccumulator == null ) {
				this.intPercentDataAccumulator = this.accumulatorFactory.safeGetIntegerPercentageDataAccumulator(metricName);
			}
			this.intPercentDataAccumulator.IIntegerAggregatingDataAccumulator_recordDataPoint(value);
		}
		this.metrics.put(metricName, System.currentTimeMillis());
	}

	/* (non-Javadoc)
	 * @see com.expd.apm.extension.jtop.metricpublisher.IMetricPublisher#publish(java.lang.String, int)
	 */
	public void publish(String metricName, int value, boolean isPercentage) {
		setPercentage(isPercentage);
		publish(metricName, value);
	}

	/* (non-Javadoc)
	 * @see com.expd.apm.extension.jtop.metricpublisher.IMetricPublisher#publish(java.lang.String, long)
	 */
	public void publish(String newMetricName, long value) {
		checkName(newMetricName);

		if(this.longFluctCountDataAccumulator == null) {
			this.longFluctCountDataAccumulator = this.accumulatorFactory.safeGetLongFluctuatingCounterDataAccumulator(metricName);
		}

		this.longFluctCountDataAccumulator.ILongCounterDataAccumulator_setValue(value);
		this.metrics.put(metricName, System.currentTimeMillis());
	}

	/* (non-Javadoc)
	 * @see com.expd.apm.extension.jtop.metricpublisher.IMetricPublisher#publish(java.lang.String, java.lang.String)
	 */
	public void publish(String newMetricName, String value) {
		checkName(newMetricName);

		if(this.stringConstantDataAccumulator == null) {
			this.stringConstantDataAccumulator = this.accumulatorFactory.safeGetStringConstantDataAccumulator(metricName, value);
		} else {
			String previousMetric = this.stringConstantDataAccumulator.IStringConstantDataAccumulator_getStringValue();
			if(previousMetric != null && !previousMetric.equals(value)) {
				this.accumulatorFactory.removeDataAccumulator(this.stringConstantDataAccumulator);
				this.stringConstantDataAccumulator = this.accumulatorFactory.safeGetStringConstantDataAccumulator(metricName, value);
			} else {
				this.stringConstantDataAccumulator = this.accumulatorFactory.safeGetStringConstantDataAccumulator(metricName, value);
			}
		}

		this.stringConstantDataAccumulator.IStringConstantDataAccumulator_getStringValue();
		this.metrics.put(metricName, System.currentTimeMillis());
	}

	public void removeMetric(String metricName) {
		try {
			this.accumulatorFactory.removeMetric(metricName);
		} catch (Exception e) {
			feedbackChannel.error(kMetricPublisher, e.getMessage(), e);
		} 
	}

	public void removeMetrics() {
		for(Iterator<Map.Entry<String, Long>> it = metrics.entrySet().iterator(); it.hasNext(); ) {
			Map.Entry<String, Long> entry = it.next();
			try {
				this.accumulatorFactory.removeMetric(entry.getKey());
			} catch (Exception e) {
				feedbackChannel.error(kMetricPublisher, e.getMessage(), e);
			} 

			it.remove();
		}
	}

	/**
	 * Gets the accumulator factory for this agentMetricPublisher.
	 *
	 * @return the accumulator factory
	 */
	public DataAccumulatorFactory getAccumulatorFactory() {
		return this.accumulatorFactory;
	}

	/**
	 * Set the accumulator factory for this agentMetricPublisher.
	 *
	 * @param accumulatorFactory the new accumulator factory
	 */
	public void setAccumulatorFactory(DataAccumulatorFactory accumulatorFactory) {
		this.accumulatorFactory = accumulatorFactory;
	}

	public Map<String, Long> getMetrics() {
		return metrics;
	}

}