package com.expd.apm.extension.agent.metricpublisher;

import com.expd.apm.extension.agent.metricpublisher.AgentMetricPublisher;
import com.expd.apm.extension.agent.metricpublisher.IMetricPublisher;
import com.expd.apm.extension.agent.metricpublisher.SystemOutMetricPublisher;
import com.wily.introscope.agent.IAgent;

/**
 * A factory for creating MetricPublisher objects.
 *
 * @author Jack Butler (jack.butler@expeditors.com)
 * @version 1.0 2013-02-11
 */
public class MetricPublisherFactory {
	private static final int SYSTEM_OUT_PUBLISHER = 0;
	private static final int AGENT_PUBLISHER = 1;
	private static MetricPublisherFactory publisherFactory = new MetricPublisherFactory();

	private static int publisherType = 0;
	private IAgent fAgent;

	/**
	 * Gets the single instance of MetricPublisherFactory.
	 *
	 * @return single instance of MetricPublisherFactory
	 */
	public static MetricPublisherFactory getInstance() {
		return publisherFactory;
	}

	/**
	 * Gets the metric publisher for this metricPublisherFactory.
	 *
	 * @return the metric publisher
	 */
	public synchronized IMetricPublisher getMetricPublisher() {
		IMetricPublisher metricPublisher = null;
		switch (publisherType) {
		case SYSTEM_OUT_PUBLISHER:
			metricPublisher = new SystemOutMetricPublisher();
			break;
		case AGENT_PUBLISHER:
			metricPublisher = new AgentMetricPublisher(this.fAgent);
			break;
		default:
			metricPublisher = new SystemOutMetricPublisher();
		}

		return metricPublisher;
	}

	/**
	 * Use system out metric publisher.
	 */
	public synchronized void useSystemOutMetricPublisher() {
		publisherType = SYSTEM_OUT_PUBLISHER;
	}

	/**
	 * Use agent metric publisher.
	 *
	 * @param agent the agent
	 */
	public synchronized void useAgentMetricPublisher(IAgent agent) {
		publisherType = AGENT_PUBLISHER;
		this.fAgent = agent;
	}
}
